NOCODB_CATEGORIES="Categories"
NOCODB_PRODUCTS="Products"
NOCODB_STATUSES="Order statuses"
NOCODB_BOT_MESSAGES="Bot messages"
NOCODB_ORDERS="Orders"

# NOCODB_CATEGORIES="Категории"
# NOCODB_PRODUCTS="Товары"
# NOCODB_STATUSES="Статусы заказов"
# NOCODB_BOT_MESSAGES="Сообщения бота"
# NOCODB_ORDERS="Заказы"